;***************************************
; BATTLEACTION.ASM
;  began 2/8/02
;  made while listening to "The Strokes"
;***************************************

; processing ACTION in battle
;  we use the actionQueue

;---------------------
; VARIABLES ACCESSIBLE
;---------------------
; - currentCharacter - type Sprite
; - currentCharPointer - pointer to beginning of 16 byte struct
; - currentCharNumber - what char is active
; - actionTimer - at 0 for no action, otherwise counts down from 10
;---------------------
; charsReadyToAttack
;   3 entry queue
;   lists the currentCharNumber of who is ready to attack
;   ordered first to last
;---------------------

; attackQueue
;  8 bytes per entry

;0 .db <char performing action>
;1 .db <victim char type>
;2 .db <action type>
;3 .db <?>
;4,5 .db <victim char ptr>
;6,7 .db <damage>

fightDamage .equ attackQueue+6
victimPointer .equ attackQueue+4

; action types
;  1 = fight

;  3 = magic
;  4 = item

 ;************************************
 ;* addAttack
 ;************************************
 ;*  adds an attack to the attack queue
 ;*  called when in readyBattleState
 ;************************************
addAttack2:
 ld de,8
 ld hl,attackQueue
 call getLastEntryOfAttackQ

 ld a,(currentCharacter)

 cp 4
 jr c,PartyMemberAttack

 ld (hl),a

 ; must be an enemy attack
 call getrand
 cp 80
 jr c,attackChar1
 cp 160
 jr c,attackChar2

 ld a,3
 ld b,80
 ld c,30
 jr addAttackTarget

attackChar1:
 ld a,1
 ld b,80
 ld c,0
 jr addAttackTarget

attackChar2:
 ld a,2
 ld b,64
 ld c,15

addAttackTarget:
 inc hl
 ld (hl),a
 push af
 inc hl
 ld a,1
 ld (hl),a
 inc hl
 inc hl
 ld (hl),b
 inc hl
 ld (hl),c
 inc hl
 pop af
 ; damage from attack

 push hl
  call findBattleChar
  ld de,13
  add hl,de
  ld b,(hl)

  push bc
   ld a,(currentCharacter)
   call findBattleChar
   ld de,12
   add hl,de
   ld a,(hl)
  pop bc

  call findHPlost
  ex de,hl
 pop hl
 
; ld de,50
 ld (hl),e
 inc hl
 ld (hl),d

 inc hl
 xor a
 ld (hl),a
 
 ; we dont care about last byte unless it is magic or an item.

 ; we need to update the state now.
 ld hl,(currentCharPointer)
 ld a,4
 ld (hl),a
 ret


PartyMemberAttack:
 ; are you the first party member to be ready?
 ld a,(charsReadyToAttack)
 ld hl,(currentCharPointer)
 inc hl
 ld b,(hl)
 cp b
 ret nz

 ; so you are ready?
 ld a,(cursorPos)
 inc a
 ld (actionSelect),a
 ret


getLastEntryOfAttackQ:
 ld a,(hl)
 or a
 ret z
 add hl,de
 jr getLastEntryOfAttackQ


 ; called when a characters gauge fills up for the first time
charAwaitsAction:
 push hl
  ld hl,(currentCharPointer)
  ld a,3								; ready for battle
  ld (hl),a

  inc hl
  ld a,(hl)
  cp 4
  jp nc,skipShowBattleOptions

  ; we fill up a 3 entry queue with the character ready to attack.  this is ordered.
  ld hl,charsReadyToAttack
  ld a,(hl)
  or a
  jr z,setReadyToAttack
  inc hl
  ld a,(hl)
  or a
  jr z,setReadyToAttack
  inc hl

setReadyToAttack:
  ex de,hl
  ld hl,(currentCharPointer)
  inc hl
  ld a,(hl)
  ld (de),a	

  ld b,a
  ld a,(charsReadyToAttack)
  cp b
  jr nz,skipShowBattleOptions

  ;****************************************
   ; setup the cursor

 ; setup the FIGHT MAGIC ITEM menu

 call setupBattleCursor

   ;****************************************

skipShowBattleOptions:
 pop hl

 ld a,255
 ret

setVictimPointer:
 ld a,(attackQueue+1)
 ld hl,battleQueue+1
 ld b,a

findVictimChar:
 ld a,(hl)
 cp b
 jr z,victimFound

 ld de,16
 add hl,de
 jr findVictimChar

victimFound:
 ; found it
 dec hl
 ld (victimCharPointer),hl
 ret


findBattleChar:
 ld de,16
 ld hl,battleQueue+1

findBattleCharL:
 ld b,(hl)
 cp b
 jr z,exitFindBattleChar

 add hl,de
 jr findBattleCharL

exitFindBattleChar:
 dec hl
 ret

;----------------------
; DEBUG for
;  findHPlost
 push af
 push bc
  ld a,(currentCharacter)
  ld l,a
  ld h,0
  xor a
  ld (currow),a
  ld (curcol),a
  B_CALL disphl
 pop bc
 pop af

 push af
  push bc
   ld l,a
	ld h,0
	ld a,1
	ld (currow),a
	ld (curcol),a
	B_CALL disphl
  pop bc
  push bc
   ld l,b
	ld h,0
	B_CALL disphl
  pop bc
 pop af

findHPlost:
 ; a = attack, b=defense

 ; hp inflicted = f(a,b)
 ; f(255,10) <= 65000
 ; f(255,255) = 9999
 ; f(100,100) = 999
 ; f(25,25) = 99
 ; f(10,255) >= 10

 ;-------
 ; f(a,b) = C*(a^2/b) + R
 ;-------
 ; c = 8
 ; R = random(0,15)
 ;------

 push bc
  ld b,a
  ld hl,0
  ld d,0
  ld e,a
squareItLoop:
  add hl,de
  djnz squareItLoop
 pop bc

 ld e,b
 ld d,0
 call DIV_HL_DE_S

 add hl,hl
 add hl,hl
 add hl,hl
 call getrand
 and 1111b
 ld c,a
 ld b,0
 add hl,bc

 ld (victimHPlost),hl
; push hl
;  ld a,2
;  ld (currow),a
;  ld (curcol),a
;  B_CALL disphl
;  B_CALL getkey
; pop hl
 ld b,h
 ld c,l
 ret

setupBattleCursor:
 xor a
 ld (actionSelect),a
 ld (cursorPos),a
 ld a,9
 ld (cursorState),a
 ld a,4			; offset by 1, b/c state 10 uses same routine
 ld (cursorMax),a	; zone for cursor
 xor a
 ld (cursorVideo),a	; reverse video (0=no,1=yes)

 ld de,plotsscreen+480	; de = where to draw cursor at
 ld hl,cursorTable

 ld b,4
setupBattleTableLoop2:
 ld (hl),e
 inc hl
 ld (hl),d
 inc hl

 push hl
  ld hl,6*12
  add hl,de
  ex de,hl
 pop hl
 djnz setupBattleTableLoop2
 ret
